/**********************************************************************************************
  
    Version 1.0   Nov 2009

    Copyright 2009 Geoff Graham - http://geoffg.net
    This program is free software: you can redistribute it and/or modify it under the terms of the GNU General
    Public License as published by the Free Software Foundation, either version 2 of the License, or (at your
    option) any later version.

    This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.  You should have received a copy of the GNU General Public License along with this program.
    If not, see <http://www.gnu.org/licenses/>.
 
**********************************************************************************************/

#include <18F2550.h> 
#include "StdDefines.h"

// configure a 20MHz crystal to operate at 48MHz (required for USB 2.0)
#fuses HSPLL,NOWDT,NOPROTECT,NOLVP,NODEBUG,USBDIV,PLL5,CPUDIV1,VREGEN
#use delay(clock=48000000)



/////////////////////////////////////////////////////////////////////////////
//
// Main configuration parameters 
//
#define NBRCHAN         85          // nbr of MHz to scan - starting at 2400 MHz
#define RSSI_SAMPLES    6           // nbr of signal level samples to average
#define RSSI_SETTLE     50          // amount of time for the signal level to settle



/////////////////////////////////////////////////////////////////////////////
//
// Define the I/O pins. 
//
#define WM_RESET    PIN_B2
#define WM_SELECT   PIN_B3
#define SPI_OUT     PIN_C7
#define SPI_IN      PIN_B0
#define SPI_CLK     PIN_B1
#define SCOPE_TRIGGER   PIN_B4

#define TRIS_A  0b11111111
#define TRIS_B  0b11100001
#define TRIS_C  0b01111111

#use fast_io(A)
#use fast_io(B)
#use fast_io(C)


/////////////////////////////////////////////////////////////////////////////
//
// CCS USB Library configuration stuff.  
//
#define USB_HID_DEVICE     FALSE
#define USB_EP1_TX_ENABLE  USB_ENABLE_BULK  //turn on EP1 for IN bulk/interrupt transfers
#define USB_EP1_RX_ENABLE  USB_ENABLE_BULK  //turn on EP1 for OUT bulk/interrupt transfers
#define USB_EP1_TX_SIZE    64               //size to allocate for the tx endpoint 1 buffer
#define USB_EP1_RX_SIZE    8                //size to allocate for the rx endpoint 1 buffer

#include ".\USB Driver\pic18_usb.h"
#include ".\USB Driver\usb_scanner_def.h"   //USB Configuration and Device descriptors for this UBS device
#include ".\USB Driver\usb.c"               //handles usb setup tokens and get descriptor reports


/////////////////////////////////////////////////////////////////////////////
//
// Global variables  
//
byte spectrum[NBRCHAN];                     // Signal strength spectrum



/////////////////////////////////////////////////////////////////////////////
//
// Define the registers used in the wireless module 
//
#define REG_ID                0x00
#define REG_CONTROL           0x03
#define REG_DATA_RATE         0x04
#define REG_CONFIG            0x05
#define REG_SERDES_CTL        0x06
#define REG_RX_INT_EN         0x07
#define REG_RX_INT_STAT       0x08
#define REG_RX_DATA_A         0x09
#define REG_RX_VALID_A        0x0A
#define REG_RX_DATA_B         0x0B
#define REG_RX_VALID_B        0x0C
#define REG_TX_INT_EN         0x0D
#define REG_TX_INT_STAT       0x0E
#define REG_TX_DATA           0x0F
#define REG_TX_VALID          0x10
#define REG_PN_CODE1          0x11
#define REG_PN_CODE2          0x12
#define REG_PN_CODE3          0x13
#define REG_PN_CODE4          0x14
#define REG_PN_CODE5          0x15
#define REG_PN_CODE6          0x16
#define REG_PN_CODE7          0x17
#define REG_PN_CODE8          0x18
#define REG_THRESHOLD_L       0x19
#define REG_THRESHOLD_H       0x1A
#define REG_WAKE_EN           0x1C
#define REG_WAKE_STAT         0x1D
#define REG_ANALOG_CTL        0x20
#define REG_CHANNEL           0x21
#define REG_RSSI              0x22
#define REG_PA                0x23
#define REG_CRYSTAL_ADJ       0x24
#define REG_VCO_CAL           0x26
#define REG_PWR_CTL           0x2E
#define REG_CARRIER_DETECT    0x2F
#define REG_CLOCK_MANUAL      0x32
#define REG_CLOCK_ENABLE      0x33
#define REG_SYN_LOCK_CNT      0x38
#define REG_MID1              0x3C
#define REG_MID2              0x3D
#define REG_MID3              0x3E
#define REG_MID4              0x3F


/////////////////////////////////////////////////////////////////////////////
// SPI read - write routines
// this was written because I could not get the CCS implementation using 18F2550 hardware to
// work reliabily.  
//
// Data is clocked at approx 1MHz with 48MHz clock, time to write/read a byte is approx 8 uSec.


// Output a byte to the SPI device
void spi_output(dout) {
    byte i;
    
    for(i = 0; i < 8; i++) {
        rotate_left(&dout, 1);              // place the MSB into bit 0 position
        if(dout & 1) output_high(SPI_OUT);  // and set output high if necessary (output is already low)
        output_high(SPI_CLK);               // strobe data on the low to high transition
        delay_cycles(2);
        output_low(SPI_CLK);                // reset the clock
        output_low(SPI_OUT);                // and set the data line low for the next bit
        }
}


// Input a byte from the SPI device
byte spi_input() {
    byte i, din = 0;
    
for(i = 0; i < 8; i++) {
        rotate_left(&din, 1);               // prepare the input variable
        output_high(SPI_CLK);               // data should be available on the low to high transition
        if(input(SPI_IN)) bit_set(din, 0);  // if we have a high input set the bit (var is initilised to 0)
        output_low(SPI_CLK);                // and reset the clock
        }
    return din;
}



/////////////////////////////////////////////////////////////////////////////
// Wireless Module (WM) registers read/write routines
//

// Write data to addr in the WM
void WriteWM(byte addr, byte data) {
    output_low(WM_SELECT);
    spi_output(addr | 0x80);
    spi_output(data);
    output_high(WM_SELECT);
}

// Read from addr in the WM
byte ReadWM(byte addr) {
    byte d;
    
    output_low(WM_SELECT);
    spi_output(addr);
    d = spi_input();
    output_high(WM_SELECT);
    return d;
}


/////////////////////////////////////////////////////////////////////////////
//
// Communicate with the wireless module to get the whole spectrum
// Copies the data into the global array spectrum[] 
//
void GetSpectrum(){
    byte cnt[32];
    byte max, maxcnt;
    byte i, j, data;

    for(i=0; i<NBRCHAN ;i++){
        WriteWM(REG_CONTROL, 0x80 + 0x10);          // set rx mode
        WriteWM(REG_CHANNEL, i);                    // set channel
        
        // clear the array used to detect the peak in the samples
        for(j = 0; j < 32; j++) cnt[j] = 0;
        max = maxcnt = 0;
        
        // read the signal strength until we have a number of readings (RSSI_SAMPLES) that are the same
        while(maxcnt < RSSI_SAMPLES){
            do {
                WriteWM(REG_CARRIER_DETECT, 0x00);  // clear the carrier detect bit
                WriteWM(REG_CARRIER_DETECT, 0x80);  // turn carrier detect back on
                delay_us(RSSI_SETTLE);              // let the reading settle
                data = ReadWM(REG_RSSI);            // read the signal level
            } while((data & 0x20) == 0);            // loop until we get a valid reading
            data = (data & 0x1F);                   // mask off the status bit
            cnt[data] += 1;                         // and add into our array of counts
            if(cnt[data] > maxcnt) { maxcnt = cnt[data]; max = data; }
        }
        WriteWM(REG_CONTROL, 0x00 + 0x10);          // clear rx mode
        spectrum[i] = max;                          // save the value for this channel
    }
}



/////////////////////////////////////////////////////////////////////////////
//
// Main loop 
//
void main(void) {
    set_tris_a(TRIS_A);
    set_tris_b(TRIS_B);
    set_tris_c(TRIS_C);
    
    setup_spi(SPI_SS_DISABLED);

    output_high(WM_SELECT);                     // Init the the wireless module (WM) select line
    output_low(SPI_CLK);                        // Init the the SPI clock line

    output_low(WM_RESET);                       // reset the wireless module (WM) until everything settles
    delay_ms(1000); output_high(WM_RESET);

    WriteWM(REG_CLOCK_MANUAL, 0x41);            // Data sheet says to
    WriteWM(REG_CLOCK_ENABLE, 0x41);
    WriteWM(REG_CONTROL, 0x10);                 // Bypass internal PLL Lock
    WriteWM(REG_ANALOG_CTL, 0x44);              // Allow writing of carrier detect register and turn on PA enable
    WriteWM(REG_CRYSTAL_ADJ, 0x40);             // Turn off 13MHz external xtal drive
    WriteWM(REG_VCO_CAL, 0xC0);                 // Set VCO cal +-5 
    WriteWM(REG_CONTROL, 0x80 + 0x10);          // set rx mode 

    usb_init_cs();

    while (TRUE) {
        usb_task();

        if(usb_enumerated()) {
            GetSpectrum();                      // scan and save the spectrum
            usb_puts(1,spectrum,NBRCHAN,100);   // and send off to the PC
        }
    }
}        
